"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const contentBuilder_1 = require("./contentBuilder");
const apiOperationParams_1 = require("./apiOperationParams");
const utils_1 = require("./utils");
const typeResolver_1 = require("./typeResolver");
const globals_1 = require("./globals");
class Fragments {
    static generateTypedProperty(name, def, required) {
        var type = typeResolver_1.TypeResolver.getPropertyType(def);
        const req = required || def.required;
        var isRequired = _.isArray(req)
            ? req.indexOf(name) > -1
            : _.isBoolean(req)
                ? req
                : globals_1.Globals.propertiesRequiredByDefault;
        return `${contentBuilder_1.ContentBuilder.toPropName(name)}${isRequired ? "" : "?"}: ${type}`;
    }
    static generateInterfaceProperty(name, def, required, prefix = "", suffix = "") {
        return `${prefix}${Fragments.generateTypedProperty(name, def, required)}${suffix}`;
    }
    static generateInterfaceProperties(def, prefix, suffix) {
        var p = [];
        utils_1.Utils.visitObject(def.properties, (name, propDef) => {
            p.push(Fragments.generateInterfaceProperty(name, propDef, def.required, prefix, suffix));
        });
        return p;
    }
    static generateInterface(name, def) {
        return `/**
The ${name} DTO/Resource used within the Api Clients
*/
interface ${name} {
${this.generateInterfaceProperties(def, "  ", ";").join("\r\n")}
}
`;
    }
    static generateModel(name, def) {
        return `/**
The ${name} Model used within the Api Clients
*/
export class ${name}Model implements ${name} {
  constructor(${this.generateInterfaceProperties(def, "public ", "").join(",")}){}
}

`;
    }
    static methodTemplate(operation, pathParams, bodyParams, params, content, paramObjectKey) {
        var id = operation["opId"] || operation.operationId;
        var parts = id.indexOf("_");
        var p;
        if (parts > -1) {
            p = id.substr(parts + 1);
        }
        else {
            p = id;
        }
        var cam = contentBuilder_1.ContentBuilder.toCamel(p);
        return `  ${operation.summary ? "/** " + operation.summary + " */" : ""}
  ${cam} = (${paramObjectKey ? `${paramObjectKey}: {` : ""}${pathParams
            .concat(bodyParams)
            .concat(params)
            .join()}${paramObjectKey ? "}" : ""}) => {
${content}
  }
`;
    }
    static getOperationReturnTypes(operation) {
        var types = {};
        _.each(operation.responses, (resp, statusCode) => {
            if (statusCode === "204") {
                types[statusCode] = typeResolver_1.TypeNamespace + "NoContent";
                return;
            }
            types[statusCode] = typeResolver_1.TypeResolver.getPropertyType(resp.schema);
        });
        return types;
    }
    static getQueryParams(operation, queryParams, paramObjectKey) {
        var paramsName = _.map(queryParams, (p) => `"${p.name}":${paramObjectKey ? `${paramObjectKey}.` : ""}${Fragments.replaceDot(p.name)}`);
        return paramsName.length
            ? `    var qs = this.buildQueryString({${paramsName.join()}});
`
            : "";
    }
    static replaceDot(val) {
        return val.replace(".", "_");
    }
    static generateApiOperation(options, path, def, callback) {
        utils_1.Utils.visitObject(def, (verb, operation) => {
            var aop = new apiOperationParams_1.ApiOperationParams(operation);
            var headers = _.map(aop.headers, (p) => p.name);
            var hasAuthHeader = headers.indexOf("Authorization") > -1;
            var params = operation.parameters
                ? _.map(aop.queries, (p) => `${Fragments.generateTypedProperty(Fragments.replaceDot(p.name), p)}`)
                : [];
            var pathParams = operation.parameters
                ? _.map(aop.paths, (p) => `${Fragments.generateTypedProperty(Fragments.replaceDot(p.name), p)}`)
                : [];
            var bodyParams = operation.parameters
                ? _.map(aop.bodies, (p) => `${p.name} : ${typeResolver_1.TypeResolver.getPropertyType(p.schema)}`)
                : [];
            var bodyParamsName = operation.parameters
                ? _.map(aop.bodies, (p) => Fragments.replaceDot(p.name))
                : [];
            var bodyParamsType = operation.parameters
                ? _.map(aop.bodies, (p) => `${typeResolver_1.TypeResolver.getPropertyType(p.schema)}`)
                : [];
            var requestType = typeResolver_1.TypeNamespace + "Void";
            if (bodyParams.length) {
                requestType = bodyParamsType[0];
            }
            var types = this.getOperationReturnTypes(operation);
            var returnType = _.filter(_.values(types), (t, i) => i === 0 || t !== "IErrorTypeNotFound").join("|");
            var qs = this.getQueryParams(operation, aop.queries, options.paramObjectKey);
            var template = `${qs}    return this.api.request<${requestType},${returnType}>("${verb}", ${hasAuthHeader ? "true" : "false"}, ${contentBuilder_1.ContentBuilder.tick}${path.replace(/\{/g, "${" + (options.paramObjectKey ? `${options.paramObjectKey}.` : ""))}${qs.length ? contentBuilder_1.ContentBuilder.dollar + "{qs}" : ""}${contentBuilder_1.ContentBuilder.tick}${bodyParamsName.length ? ", " : ""}${bodyParamsName
                .map((pm) => options.paramObjectKey ? `${options.paramObjectKey}.${pm}` : pm)
                .join()})`;
            template = Fragments.methodTemplate(operation, pathParams, bodyParams, params, template, options.paramObjectKey);
            var idx = operation.operationId.indexOf("_");
            var parts = idx > -1 ? operation.operationId.substr(0, idx) : operation["tags"][0];
            callback(parts, template);
        });
    }
    static getDuplicateGroups(items, groupBy) {
        return _.reject(_.groupBy(items, groupBy), (g, k) => g.length === 1);
    }
    static ensureUniqueMethodNames(swagger) {
        var operationsList = [];
        utils_1.Utils.visitObject(swagger.paths, (name, def) => {
            utils_1.Utils.visitObject(def, (verb, op) => {
                operationsList.push({ op, verb });
            });
        });
        var duplicates = this.getDuplicateGroups(operationsList, (o) => o.op.operationId);
        _.each(duplicates, (ops) => {
            var duplicateGroups = this.getDuplicateGroups(ops, (o) => o.op.operationId + o.verb);
            if (!duplicateGroups.length) {
                _.each(ops, (op, idx) => {
                    op.op["opId"] = op.op.operationId + contentBuilder_1.ContentBuilder.toProper(op.verb);
                });
                return;
            }
            _.each(ops, (op, idx) => {
                let ext;
                if (op.verb === "get") {
                    ext = op.op.parameters
                        .map((c) => c.in === "path" ? contentBuilder_1.ContentBuilder.toProper(c.name) : "")
                        .join("");
                }
                else if (op.verb === "post") {
                    ext = op.op.parameters
                        .map((c) => contentBuilder_1.ContentBuilder.toProper(c.name))
                        .join("");
                }
                else if (op.verb === "put") {
                    ext = op.op.parameters
                        .map((c) => contentBuilder_1.ContentBuilder.toProper(c.name))
                        .join("");
                }
                else {
                    ext = `${idx}`;
                }
                op.op["opId"] = op.op.operationId + ext;
            });
        });
    }
    static createOperationDictionary(swagger, options) {
        var operations = {};
        this.ensureUniqueMethodNames(swagger);
        utils_1.Utils.visitObject(swagger.paths, (name, def) => {
            Fragments.generateApiOperation(options, name, def, (id, content) => {
                if (!operations[id]) {
                    operations[id] = [content];
                }
                else {
                    operations[id].push(content);
                }
            });
        });
        return operations;
    }
    static buildApiClients(operations) {
        var clients = {};
        _.each(operations, (ops, key) => {
            clients[key] = `/**
The ${key} Api Client - implements all operations from this logical endpoint
*/
export class ${key}Client extends ApiClientBase {
  constructor(public api: IApiClient){
    super();
  }
${ops.join("")}
}

`;
        });
        return clients;
    }
    static generateEnum(name, members, constEnum) {
        return `${constEnum ? "const " : ""}enum ${name} {
  ${members.map((m) => `${m.name} = ${m.value}`).join(",\n  ")}
}
`;
    }
    static getDtoItems(swagger) {
        var dtoSB = [];
        utils_1.Utils.visitObject(swagger.definitions, (name, def) => {
            dtoSB.push(Fragments.generateInterface(contentBuilder_1.ContentBuilder.toTypeName(name), def));
        });
        return dtoSB;
    }
    static getEnums(constEnum) {
        var dtoSB = [];
        var enumTypeNames = _.keys(typeResolver_1.TypeResolver.enumTypes);
        _.each(enumTypeNames, (name) => {
            dtoSB.push(Fragments.generateEnum(name, typeResolver_1.TypeResolver.enumTypes[name], constEnum));
        });
        return dtoSB;
    }
}
exports.Fragments = Fragments;
