"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const contentBuilder_1 = require("./contentBuilder");
const fragments_1 = require("./fragments");
const typeResolver_1 = require("./typeResolver");
class TemplateBuilder {
    static writeApiInterface() {
        return `/**
The main Api Client implementation required to provide execution of the API (http verb based) requests
Implement your transport (window.fetch [whatwg], jquery ajax, etc)
Handle authentication tokens or cookies
*/
export interface IApiClient {
  /**
  Send a request to the Api
  @param method The Http Method
  @param authorization Is authorization required?
  @param url The Request Url
  @param payload The Request Payload
  */
  request<T,R>(method: string, authorization: boolean, url: string, payload?: T) : Promise<IFetchApiResponse<R>>;
}

`;
    }
    static writeFetchApiInterface() {
        return `
export interface IFetchApiFailure{
  text?: string;
  json?: any;
}

export interface IFetchApiResponse<T>{
  payload?: T;
  response: {status: number, ok?: boolean};
  failure?: IFetchApiFailure;
}

`;
    }
    static wrapUriParamWithEncoder(variableName, options) {
        if (!options.encodeUriParams) {
            return variableName;
        }
        return `encodeURIComponent(${variableName})`;
    }
    static writeApiClientBase(options) {
        return `/**
The base class used for all Api Clients
*/

abstract class ApiClientBase {
  protected buildQueryString(values: {}) {
    var paramsName = Object.keys(values).filter(
      (k) => values[k] !== undefined && values[k] !== null
    );
    if (!paramsName.length) {
      return "";
    }

    return ${contentBuilder_1.ContentBuilder.tick}?${contentBuilder_1.ContentBuilder.dollar}{paramsName
      .map((pn) => this.buildParam(pn, values[pn]))
      .join("&")}${contentBuilder_1.ContentBuilder.tick};
  }

  private buildParam(pn: string, value: any) {
    if (Array.isArray(value)) {
      return value.map((v) => ${contentBuilder_1.ContentBuilder.tick}${contentBuilder_1.ContentBuilder.dollar}{pn}=${contentBuilder_1.ContentBuilder.dollar}{v}${contentBuilder_1.ContentBuilder.tick}).join("&");
    }
    return ${contentBuilder_1.ContentBuilder.tick}${contentBuilder_1.ContentBuilder.dollar}{pn}=${contentBuilder_1.ContentBuilder.dollar}{value}${contentBuilder_1.ContentBuilder.tick};
  }
}
`;
    }
    static writeTypeNotFound() {
        return `/**
A generated interface - usage in the generated endpoints will indicate the system was unable to detect a type
*/
export interface IErrorTypeNotFound {
}
`;
    }
    static writeApiFactory(operations) {
        return `/**
The Api Factory allows you to construct a single object and have access to all the generated Api Clients
*/
export class ApiFactory {
  constructor(private api: IApiClient){}
${contentBuilder_1.ContentBuilder.items(operations, (classList, key) => `
  /** The ${key} Api Client instance */
  ${contentBuilder_1.ContentBuilder.toCamel(key)}: ${key}Client = new ${key}Client(this.api);`)}
}
`;
    }
    static writeFetchApiImp() {
        return `/**
Sample implementation of IApiClient using ES6 fetch
*/

export class FetchApiClient implements IApiClient{
  constructor(private baseUrl: string, private fetchAuthToken: () => string){}

  request<T,R>(method: string, authorization: boolean, url: string, payload?: T) : Promise<IFetchApiResponse<R>>{
    var init: RequestInit = FetchApiClient.requestInit(method, payload ? JSON.stringify(payload) : undefined);
    if (authorization) {
        FetchApiClient.authorizeInit(init, this.fetchAuthToken())
    }

    return FetchApiClient.fetchJson<R>(this.baseUrl + url, init);
  }

  static fetchJson<R>(url: string, init: RequestInit) : Promise<IFetchApiResponse<R>> {
    return window.fetch(url, init).then<R>(r => {
      if (r.status === 204 ){
        return {response: r};
      }
      return r.json().then(p => {
        return {response: r, payload: p};
      })
    });
  }

  static authorizeInit(init: RequestInit, token: string) {
    if (token) {
      init.headers['Authorization'] = ${contentBuilder_1.ContentBuilder.tick}Bearer ${contentBuilder_1.ContentBuilder.dollar}{token}${contentBuilder_1.ContentBuilder.tick};
    } else {
      console.log("Api request required Authorization but no token was found!")
    }
  }

  static requestInit(method: string = "get", payload: string = null, contentType: string = "application/json"): RequestInit {
    var init: RequestInit =  {
      method,
      headers: {
        'Content-Type': contentType
      }
    };
    if (payload) {
      init.body = payload;
    }

    return init;
  }

  loginFormPassword<R>(url: string, username: string, password: string): Promise<R> {
    return FetchApiClient.fetchJson<R>(this.baseUrl + url, FetchApiClient.loginPasswordInit(username, password));
  }

  static loginPasswordInit(username: string, password: string): RequestInit {
    return FetchApiClient.requestInit(
      "post",
      ${contentBuilder_1.ContentBuilder.tick}username=${contentBuilder_1.ContentBuilder.dollar}{encodeURIComponent(username)}&password=${contentBuilder_1.ContentBuilder.dollar}{encodeURIComponent(password)}&grant_type=password${contentBuilder_1.ContentBuilder.tick},
      "application/x-www-form-urlencoded");
  }
}
`;
    }
    static run(options, swagger, callback) {
        var operations = fragments_1.Fragments.createOperationDictionary(swagger, options);
        const ops = _.keys(operations);
        var opCount = _.reduce(_.values(operations), function (memo, num) {
            return memo + num.length;
        }, 0);
        console.log(`Found ${ops.length} Operation Groups (total of ${opCount} Operations)`);
        console.log(` >> `, ops.join(","));
        var apiClients = fragments_1.Fragments.buildApiClients(operations);
        var definitions = `declare module ${typeResolver_1.TypeNamespaceName}{
${fragments_1.Fragments.getDtoItems(swagger).join("")}

${fragments_1.Fragments.getEnums(!options.fullEnums).join("")}

${swagger.definitions["Void"]
            ? ""
            : "/** Indicate a Void return type */\r\ninterface Void {}"}

${swagger.definitions["NoContent"]
            ? ""
            : "/** Indicate a NoContent (204) return type */\r\ninterface NoContent {}"}
}`;
        var content = `${options.fetchApiInterfaceImport || ""}

${this.writeApiInterface()}
${options.fetchApiInterfaceImport ? "" : this.writeFetchApiInterface()}

${options.generateFetchClient ? this.writeFetchApiImp() : ""}

${this.writeApiClientBase(options)}

${this.writeTypeNotFound()}

${_.values(apiClients).join("")}

${this.writeApiFactory(operations)}
`;
        callback([definitions, content]);
    }
}
exports.TemplateBuilder = TemplateBuilder;
